unit UnitFormListBase;
{
When creating a new list sub-class from this form (using form inheritance --
go to File|New..., choose the "CustomerOrders" tab, then choose this form) and
override the abstract methods.

  procedure Initializeform; override;
    This is where you do whatever initializations you need.

  procedure Finalizeform; override;
    This is where you do whatever finalizations you need.

  procedure ClearWhereClause; virtual; // abstract
    This is where you set the UI to its default settings.

  procedure UpdateWhereClause;
    This is where you string together the where clause and assign it to the
    associated object.

  procedure OpenNewEntity; override;
    This is a little awkard: you have to fetch a new business object instance,
    then use its ID to open the form. Once the form is open you free the local
    reference to the newly created business object.

  function ObjectClass: TObjectBaseClass; override;
  	The ancestor uses this information to create an instance of the list's
    associated ObjectList.

  function OpenSelectedFormType: TFormBaseClass; override;
  	The ancestor uses this information to open the form for the selected record.

  function SelectedKey: integer; override;
    SelectedKey returns the key for the current record.


Don't forget to add an action to open the list from UnitFormMain.

}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Menus, Db, Grids, DBGrids, ActnList, ComCtrls, ToolWin,
  ImgList, UnitObjectBase, Buttons, UnitFormBase, UnitFrameResultSetBase;

type
  TFormListBase = class;
  TFormListBaseClass = class of TFormListBase;
  TListSelectEvent = procedure(aFormList: TFormListBase) of object;

  TFormListBase = class(TFormBase)
    PanelListBaseBottom: TPanel;
    ButtonOkApply: TButton;
    ButtonCancelClose: TButton;
    PopupMenu: TPopupMenu;
    ActionList: TActionList;
    ActionApply: TAction;
    ActionClose: TAction;
    Open1: TMenuItem;
    PanelListBaseClientBackground: TPanel;
    ActionFind: TAction;
    Find1: TMenuItem;
    ToolBar1: TToolBar;
    PanelFilter: TPanel;
    BitBtnFind: TBitBtn;
    BitBtn2: TBitBtn;
    ToolButton1: TToolButton;
    ToolButton4: TToolButton;
    PanelListBaseApplyClose: TPanel;
    PanelListBaseGridBackground: TPanel;
    StatusBarAbstract: TStatusBar;
    ButtonListBaseApply: TButton;
    ButtonListBaseClose: TButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ActionOpen: TAction;
    procedure ActionApplyExecute(Sender: TObject);
    procedure ActionFindExecute(Sender: TObject);
    procedure ActionCloseExecute(Sender: TObject);
    procedure ActionOpenExecute(Sender: TObject);
    procedure ActionOpenUpdate(Sender: TObject);
  private
    FOnListSelect: TListSelectEvent;
  protected
    procedure Apply;
    procedure Initializeform; override;
    procedure ClearWhereClause; virtual; // abstract
    function GetFrameResultSet: TFrameResultSetBase; virtual; abstract;
  public
    property OnListSelect: TListSelectEvent read FOnListSelect write FOnListSelect;
    function ShowModalPickListOKCancel: integer;
    function ShowModalPickListApplyClose: integer;
  end;

implementation

{$R *.DFM}

var ClassVariableListOfSingletonLists: TList;

{ TFormList }

procedure TFormListBase.Apply;
begin
  if Assigned(OnListSelect)
    then OnListSelect(Self);
end;

//------------------------------------------------------------------------------
function TFormListBase.ShowModalPickListApplyClose: integer;
begin
  ActionApply.Caption := 'Apply';
  ButtonOkApply.ModalResult := mrNone;
  ButtonCancelClose.Caption := 'Close';
  PanelListBaseBottom.Visible := TRUE;
  StatusBarAbstract.SizeGrip := FALSE;
  Result := Self.ShowModal;
end;

function TFormListBase.ShowModalPickListOKCancel: integer;
begin
  ActionApply.Caption := 'OK';
  ButtonOkApply.ModalResult := mrOK;
  ButtonCancelClose.Caption := 'Cancel';
  StatusBarAbstract.SizeGrip := FALSE;
  PanelListBaseBottom.Visible := TRUE;
  Result := Self.ShowModal;
end;
//------------------------------------------------------------------------------

procedure TFormListBase.ActionApplyExecute(Sender: TObject);
begin
  inherited;
  Apply;
end;

procedure TFormListBase.Initializeform;
begin
  inherited;
end;

procedure TFormListBase.ActionFindExecute(Sender: TObject);
begin
  inherited;
  GetFrameResultSet.ResultSetObject.RefreshResultSet;
end;

procedure TFormListBase.ClearWhereClause;
begin
end;

procedure TFormListBase.ActionCloseExecute(Sender: TObject);
begin
  inherited;
  ModalResult := mrCancel;
end;

procedure TFormListBase.ActionOpenExecute(Sender: TObject);
begin
  inherited;
  GetFrameResultSet.OpenEntityForm;
end;

procedure TFormListBase.ActionOpenUpdate(Sender: TObject);
begin
  inherited;
  ActionOpen.Enabled := not(GetFrameResultSet.ResultSetObject.IsEmpty);
end;

initialization

  ClassVariableListOfSingletonLists := TList.Create;

finalization

  ClassVariableListOfSingletonLists.Free;

end.
